package de.hdm_stuttgart.mi.sd1.store;

import java.util.Arrays;

/**
 * A collection of methods dealing with arrays
 *
 */
public class Arraymethods {

  /**
   * 
   * @param values      The array to be searched
   * @param searchValue The value to be found
   * 
   * @return            The index of the first occurrence of searchValue
   *                    in values if exists, a value smaller than zero otherwise
   */
  public static int findIndex(int[] values, int searchValue) {
    for (int i = 0; i < values.length; i++) {
      if (searchValue == values[i]) {
        return i;
      }
    }
    return -1; // searchValue does not exist in values
  }

  /**
   * Check whether two arrays contain identical values in identical
   * order. Thus the arrays [-7, 1] and [1, -7] shall be regarded as
   * being different.
   *  
   * @param a the first array to be compared.
   * @param b the second array to be compared.
   * @return true if a[] and b[] are equal, false otherwise.
   */
  public static boolean isEqual(int a[],  int b[]) {
    if (a.length == b.length) {
      for (int i = 0; i < a.length; i++) {
        if (a[i] != b[i]) {
          return false;
        }
      }
      return true; // All array entries are equal
    } else { // Arrays do have different size and thus differ
      return false;
    }
  }

  /**
   * Swap the contents of two arrays. If both arrays differ in size
   * exchange only the "common" portions. Thus if we have e.g. 
   * a = [ 3, 5, -3] and
   * b = [-1, 4] 
   * 
   * swapping should only affect the first two positions:
   * 
   * a = [-1, 4, -3] and
   * b = [ 3, 5] 
   * 
   * @param a The first array to be swapped
   * @param b The second array to be swapped
   * 
   * @return The number of elements being swapped. This is equal
   *         to the minimum of both arrays' length.
   */
  public static int swap(int a[], int b[]) {
    final int swapCount = Math.min(a.length, b.length);
    for (int i = 0; i < swapCount; i++) {
      final int tmp = a[i];
      a[i] = b[i];
      b[i] = tmp;
    }
    return swapCount;
  }

  /**
   * Create a new array by appending all values of b[]
   * at the end of a[]. Thus e.g. a=[1, -7, 1] and b=[0, 5]
   * shall be concatenated to [1, -7, 1, 0, 5].
   * 
   * @param a The first array.
   * @param b The second array.
   * @return The concatenation of all values from a and b.
   */
  public static int[] concat(int a[], int b[]) {
    final int[] returnValue = new int[a.length + b.length];
    for (int i = 0; i < a.length; i++) {
      returnValue[i] = a[i];
    }
    for (int i = 0; i < b.length; i++) {
      returnValue[a.length + i] = b[i];
    }
    return returnValue;
  }

  /**
   * Check whether a given string is a palindrome or not. Test cases
   * are being supplied at http://www.palindromelist.net
   * 
   * @param candidate The possible palindrome string.
   * @return True if candidate is a palindrome with letter case
   *         being ignored, false otherwise. Non-letter characters like space,
   *         ?,;:. will be ignored as well.
   */
  public static boolean isPalindrome(final String candidate) {
    
    final char[] cArray = new char[candidate.length()];
    int numCandidateLetters = 0;
    
    for (int i = 0; i < candidate.length(); i++) { // Strip off non-letters
      final char candidateChar = candidate.toLowerCase().charAt(i);
      if (Character.isAlphabetic(candidateChar)) {
        cArray[numCandidateLetters++] = candidateChar;
      }
    }
    // cArray holds the "real" palindrome candidate with
    // non-netters being stripped of having
    // index range [0, numCandidateLetters - 1]. So let's check it!
    //
    for (int i = 0; i < numCandidateLetters / 2; i++) {
      if (cArray[i] != cArray[numCandidateLetters - 1 - i]) {
        return false;
      }
    }
    return true;
  } 

  /**
   * 
   * Check whether two arrays contain identical values having
   * identical multiplicity irrespective of ordering. Thus the
   * arrays [-7, 1] and [1, -7] shall be regarded as
   * equivalent whereas [7, 1, 1] and [7, 1, 7] are divergent due to
   * different multiplicity of elements being contained.
   *  
   * @param a the first array to be compared.
   * @param b the second array to be compared.
   * @return true if a[] and b[] contain the same elements having identical multiplicity, false otherwise.
   */
  public static boolean containsSameElements(int a[], int b[]) {

    if (a.length == b.length) {
      final int[] bCopy = Arrays.copyOf(b, b.length);// We need a copy to be modified,
      // do not touch the orginal!
      int bcopyRemainingElements = b.length;
      for (int i = 0; i < a.length; i++) {
        final int indexInBcopy =  findIndex(bCopy, a[i]);
        if (indexInBcopy < 0) {
          return false;     // Element of a[] not found in b[]
        } else {
          if (indexInBcopy < bcopyRemainingElements - 1) {        // "Discard" element from bCopy[]
            bCopy[indexInBcopy] = bCopy[--bcopyRemainingElements];
          }
        }
      }
      return true;
    } else {// Arrays differ in length and thus cannot be equivalent
      return false;
    }
  }
}
