import greenfoot.Greenfoot;
import greenfoot.GreenfootImage;
import greenfoot.World;

import java.awt.Color;

/**
 * A   <a href="http://en.wikipedia.org/wiki/Conway's_Game_of_Life" target="_blank">game of live</a> world consisting of lines and cells.
 * 
 */
public class LifeWorld extends World {

  private static int  initialLivingPercentage = 25;

  private final GreenfootImage gridImage;

  private static final int width = 50;
  private static final int height = 50;
  private static final int cellSize = 10;

  /**
   * The probability for a newly born cell
   * to be in 'living' ( See {@link Cell#setState(CellState)}) state.
   * 
   * @param initialLivingPercentage A value from [0, 100] ranging from
   *        'all cells dead' to 'all cells alive'
   */
  public static void setInitialLivingPercentage(int initialLivingPercentage) {
    LifeWorld.initialLivingPercentage = initialLivingPercentage;
  }

  /** Randomly return the two states {@link CellState#living} and {@link CellState#dead}. The
   *  corresponding probabilities depend on the value of {@link #setInitialLivingPercentage(int)}.
   * 
   * @return Either {@link CellState#living} or {@link CellState#dead}.
   */
  public static CellState getRandomLifeOrDeadState() {
    return Greenfoot.getRandomNumber(100) < initialLivingPercentage ? 
        CellState.living : CellState.dead;
  }

  /**
   * Constructor for objects of class Grid. Calls methods to draw the grid
   * lines, to create the array of randomly initialized cells .
   * 
   */
  public LifeWorld() {
    super(width, height, cellSize);
    gridImage = getBackground();
    drawGrid();
    createCells();  // Create an initialize array all cells.
  }

  /**
   * Draw the cell grid's lines.
   */
  private void drawGrid() {
    gridImage.setColor(Color.red);

    final int endX = cellSize * width,
              endY = cellSize * height;

    for (int x = 0; x < endX; x += cellSize) { // Drawing vertical grid lines. Diagram
      gridImage.drawLine(x, 0, x, endY);       // is being scaled by cellSize:
    }                                          // 
                                               //            0   1   2 ...(width - 1)
                                               //         0  |   |   | ...  |
                                               //         1  |   |   | ...  |
                                               //         2  |   |   | ...  |
                                               //           ... ... ...    ...   
                                               // (height-1) |   |   | ...  |
    
    for (int y = 0; y < endY; y += cellSize) { // Drawing horizontal grid lines. Diagram 
      gridImage.drawLine(0, y, endX, y);       // is being scaled by cellSize:
    }                                          //
                                               //            0   1   2 ...(width - 1)
                                               //         0  __________..._____
                                               //         1  __________..._____ 
                                               //         2  __________..._____
                                               //           .....................
                                               // (height-1) __________..._____
  }

  /**
   * Turn (x|y) coordinates into linear cell array index 
   * values ranging from 0 to (width * height - 1).
   * 
   * Consider a simplified example of an array having width = 4 and
   * height = 3:
   * 
   *   {(0|0) (1|0) (2|0) (3|0)} --> Linear array Index values {0, 1, 2, 3}
   *   {(0|1) (1|1) (2|1) (4|1)} --> Linear array Index values {4, 5, 6, 7}
   *   {(0|2) (1|2) (2|2) (4|2)} --> Linear array Index values {8, 9, 10, 11}
   * 
   * @param x horizontal position in cell coordinates
   * @param y vertical position in cell coordinates
   * @return The linear array index.
   */
  private int getCellIndex(int x, int y) {
    return x + y * width;
  }

  /** Create the array of cells and randomly turn some of them on
   *  according to {@link #setInitialLivingPercentage(int)}.
   */
  private void createCells() {
    final Cell [] cellsToReturn =             // Variable name has to be different from LifeWorld.cells
        new Cell[width * height];             // in order to initialize final variable.
    for (int x = 0; x < width; x++) {
      for (int y = 0; y < height; y++) {
        final Cell c = new Cell(cellSize, getRandomLifeOrDeadState());
        cellsToReturn[getCellIndex(x, y)] = c; // A separate array allows for easier access than via
                                               // Greenfoot's own "world" object accounting mechanism.
        addObject(c, x, y);                    // Add to Greenfoot world.
      }
    }
  }
}
