import greenfoot.Actor;
import greenfoot.GreenfootImage;

/**
 * Implementing a single cell. Each cell has its own state information
 * and a reference to its surrounding neighbors,
 *
 */
public class Cell extends Actor {

  CellState state, previousState;
  final GreenfootImage myBody;
  final int myBodySize;
  Cell[] neighbors;

  /**
   * Setting the current (C) cell's neighbors (N). A cell may have
   * a maximum of eight , five or just three neighbors depending on its
   * relative position to the world's boundary:
   * 
   * .........   _________     _____
   * ...NNN...   ...NCN...    |CN...
   * ...NCN...   ...NNN...    |NN...
   * ...NNN...   .........    |.....
   * .........
   *    
   * @param neighbors The cell's set of neighbors  
   */
  public void setNeighbors(Cell[] neighbors) {
    this.neighbors = neighbors;
  }

  /**
   * Create the cell body's image and initialize state.
   * 
   * @param cellSize The cell's size in pixel units,
   * @param intialState Indicating {@link CellState#ALIVE} or {@link CellState#DEAD} 
   */
  public Cell(int cellSize, CellState intialState) {
    super();
    state = previousState = intialState;
    myBody = new GreenfootImage(cellSize, cellSize);
    setImage(myBody);
    myBodySize = cellSize - 2;
    displayState();
  }

  /**
   * On first execution, find and save neighboring cell references. After that,
   * apply cell rule and show cell state.
   */
  public void act() {
    applyRule();
    displayState();
  }

  /**
   * Save the previous state in preparation for the next time step.
   */
  public void savePreviousState() {
    previousState = state;
  }

  /**
   * Set the state of the cell.
   * @param state See {@link #getState()}}
   */
  public void setState(final CellState state) {
    this.state = state;
    displayState();
  }

  /**
   * Get the state of the cell.
   * @return Either {@link CellState#ALIVE} or {@link CellState#DEAD}.
   */
  public CellState getState() {
    return state;
  }

  /**
   * Flip the cell's state.
   */
  public void flipState() {
    setState(state.getInverseState());
  }

  /**
   * Count neighboring cells currently living.
   * 
   * @return The number of living cells among the set of neighbors
   *          according to {@link #setNeighbors(Cell[])}
   */
  public int getLivingNeighborCount() {
    int sum = 0;
    for (final Cell neighbor: neighbors) {
      if (CellState.ALIVE == neighbor.previousState) {
        sum ++;
      }
    }
    return sum;
  }

  /**
   * Apply the Game of Life rules to the current cell situation.
   */
  public void applyRule() {
    final int livingNeighborsCount = getLivingNeighborCount();
 
    if (CellState.ALIVE == previousState) { // Living Cell
      if (livingNeighborsCount < 2 ||        // Dying on account of under-population
          3 < livingNeighborsCount) {        // Dying on account of overcrowding
        state = CellState.DEAD;
      } else {                               // two or three living neighbors
        state = CellState.ALIVE;
      }
    } else {                                 // cell is dead
        if (3 == livingNeighborsCount) {     // Born by reproduction
          state = CellState.ALIVE;
        } else {                             // remain dead
          state = CellState.DEAD;  
        }
    }
  }

  /**
   * Draw the cell's body based on its state.
   */
  private void displayState() {
    myBody.setColor(state.color);
    myBody.fillRect(1, 1, myBodySize + 1, myBodySize + 1);
  }
}
