package de.hdm_stuttgart.de.sd1.math;

/**
 * Hint: Embedded formulas can only be viewed correctly when
 * using Firefox or Safari web browsers.
 * 
 * This class is used to illustrate the implementation of a subset of functions from
 * {@link java.lang.Math} by means of their corresponding power series.
 *
 */
public class Math {
  
  private static final double PI = java.lang.Math.PI;
  
  static int seriesLimit = 1;
  
  /**
   *  
   * @param seriesLimit The number of terms of a power series to be included.
   * 
   */
  public static void setSeriesLimit(int seriesLimit) {
    Math.seriesLimit = seriesLimit;
  }
  
  /**
   * Approximating the natural exponential function by a finite
   * number of terms from the corresponding power series:
   * 
   * {@html.extend
<math display="block" xmlns="http://www.w3.org/1998/Math/MathML">
  <mtable>
    <mtr>
      <mtd>
        <mrow>
          <msup>
            <mi>e</mi>
            <mi>x</mi>
          </msup>
          <mo>=</mo>
          <mrow>
            <mi>1</mi>
            <mo>+</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>1</mi>
              </msup>
              <mi>1!</mi>
            </mfrac>
            <mo>+</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>2</mi>
              </msup>
              <mi>2!</mi>
            </mfrac>
            <mo>+</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>3</mi>
              </msup>
              <mi>3!</mi>
            </mfrac>
            <mo>+</mo>
            <mn>...</mn>
            <mo>+</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>seriesLimit</mi>
              </msup>
              <mrow>
                <mi>seriesLimit</mi>
                <mi>!</mi>
              </mrow>
            </mfrac>
          </mrow>
        </mrow>
      </mtd>
    </mtr>
    <mtr>
      <mtd>
        <mrow>
          <mo>=</mo>
          <mrow>
            <munderover>
              <mo>∑</mo>
              <mrow>
                <mi>i</mi>
                <mo>=</mo>
                <mi>0</mi>
              </mrow>
              <mi>seriesLimit</mi>
            </munderover>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>i</mi>
              </msup>
              <mrow>
                <mi>i</mi>
                <mi>!</mi>
              </mrow>
            </mfrac>
          </mrow>
        </mrow>
      </mtd>
    </mtr>
  </mtable>
</math>}
   * 
   * A power series implementation has to be finite since an 
   * infinite number of terms requires infinite execution time.
   * 
   * The number of terms to be considered can be set by {@link #setSeriesLimit(int)}}
   * 
   * @param x The exponential's argument as in  {@html.extend
<math display="inline" xmlns="http://www.w3.org/1998/Math/MathML">
  <mrow>
    <msup>
      <mi>e</mi>
      <mi>x</mi>
    </msup>
  </mrow>
</math>}

   * @return The value {@html.extend
<math display="inline" xmlns="http://www.w3.org/1998/Math/MathML">
  <mrow>
    <msup>
      <mi>e</mi>
      <mi>x</mi>
    </msup>
  </mrow>
</math>} itself.
   */
  public static double exp(double x) {
    double currentTerm = 1.,  // the first (i == 0) term x^0/0!
        sum = currentTerm;    // initialize to the power series' first term
    
    for (int i = 1; i <= seriesLimit; i++) { // i = 0 has already been completed.
      currentTerm *= x / i;
      sum += currentTerm;
    }
    return sum;
  }
  
  /**
   * Calculating the sine by means of the corresponding power series: 
   {@html.extend
<math display="block" xmlns="http://www.w3.org/1998/Math/MathML">
  <mtable>
    <mtr>
      <mtd>
        <mrow>
          <mrow>
            <mi>sin</mi>
            <mo>⁡</mo>
            <mrow>
              <mo>(</mo>
              <mi>x</mi>
              <mo>)</mo>
            </mrow>
          </mrow>
          <mo>=</mo>
          <mrow>
            <mi>x</mi>
            <mo>-</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>3</mi>
              </msup>
              <mrow>
                <mi>3</mi>
                <mo>!</mo>
              </mrow>
            </mfrac>
            <mo>+</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>5</mi>
              </msup>
              <mrow>
                <mi>5</mi>
                <mo>!</mo>
              </mrow>
            </mfrac>
            <mo>-</mo>
            <mfrac>
              <msup>
                <mi>x</mi>
                <mi>7</mi>
              </msup>
              <mrow>
                <mi>7</mi>
                <mo>!</mo>
              </mrow>
            </mfrac>
            <mo>+</mo>
            <mi>...</mi>
          </mrow>
        </mrow>
      </mtd>
    </mtr>
    <mtr>
      <mtd>
        <mrow>
          <mo>=</mo>
          <mrow>
            <munderover>
              <mo>∑</mo>
              <mrow>
                <mi>i</mi>
                <mo>=</mo>
                <mi>0</mi>
              </mrow>
              <mi>seriesLimit</mi>
            </munderover>
            <mrow>
              <msup>
                <mrow>
                  <mo>(</mo>
                  <mi>-1</mi>
                  <mo>)</mo>
                </mrow>
                <mrow>
                  <mi>2</mi>
                  <mi>i</mi>
                </mrow>
              </msup>
              <mfrac>
                <msup>
                  <mi>x</mi>
                  <mrow>
                    <mi>2</mi>
                    <mo>i</mo>
                    <mo>+</mo>
                    <mi>1</mi>
                  </mrow>
                </msup>
                <mrow>
                  <mrow>
                    <mo>(</mo>
                    <mi>2</mi>
                    <mi>i</mi>
                    <mo>+</mo>
                    <mi>1</mi>
                    <mo>)</mo>
                  </mrow>
                  <mo>!</mo>
                </mrow>
              </mfrac>
            </mrow>
          </mrow>
        </mrow>
      </mtd>
    </mtr>
  </mtable>
</math>} 
   * 
   * @param x The sine's argument as in  {@html.extend
<math display="inline" xmlns="http://www.w3.org/1998/Math/MathML">
  <mrow>
    <mi>sin</mi>
    <mo>⁡</mo>
    <mrow>
      <mo>(</mo>
      <mi>x</mi>
      <mo>)</mo>
    </mrow>
  </mrow>
</math>}
   *
   * @return The value {@html.extend
<math display="inline" xmlns="http://www.w3.org/1998/Math/MathML">
  <mrow>
    <mi>sin</mi>
    <mo>⁡</mo>
    <mrow>
      <mo>(</mo>
      <mi>x</mi>
      <mo>)</mo>
    </mrow>
  </mrow>
</math>} itself.
*/
  public static double sin(double x) {
    // Step 1: Normalize x to [-PI, +PI[
    final long countTimes2PI = (long) java.lang.Math.rint(x / 2 / PI);
    if (countTimes2PI != 0) {
      x -= 2 * PI * countTimes2PI;
    }
    
    // Step 2: Normalize x to [-PI/2, +PI/2]
    // Since sin(x) = sin (PI - x) we continue to normalize
    // having x in [-PI/2, +PI/2]
    if (PI/2 < x) {
      x = PI - x;
    } else if (x < -PI/2) {
      x = -x - PI;
    }

    // Step 3: Continue with business as usual
    
    double currentTerm = x,   // the first (i == 0) term x^1/1!
        sum = currentTerm;    // initialize to the power series' first term

    // Slightly incorrect if seriesLimit is even
    for (int i = 1; i <= seriesLimit; i += 2) { // Caution: Steps of two!!
      final double previousTerm =               // save the current term
          currentTerm *= x * x / (2 * i) / (2 * i + 1) ;
      currentTerm *= x * x / (2 * i + 2) / (2 * i + 3);
      sum += currentTerm - previousTerm;      //Summing up reordered
    }
    return sum;
  }}
