package sax.ns.stat;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** Creating statistical data. Group elements occurring in arbitrary
 * document instances by their respective namespaces and count the 
 * number of occurrences.
 */
public class FrequencyGatherer extends DefaultHandler {

  private Map<String, ElementcountByName> frequenciesByNamespace =
      new HashMap<String, ElementcountByName>();
  
  @Override
  public void startElement(final String uri, final String localName, final String qName,
      Attributes attributes) throws SAXException {

    ElementcountByName elementsInNamespace = frequenciesByNamespace.get(uri);
    if (null == elementsInNamespace) {// Does an entry to the current namespace yet exist?
      elementsInNamespace = new ElementcountByName();       // Not yet, so create 
      frequenciesByNamespace.put(uri, elementsInNamespace); // and add a new entry.
    }
    elementsInNamespace.increment(localName);
  }
  
  /**
   * Printing element names with their respective number of occurrences grouped
   * by namespace affiliation to standard output.
   * 
   * Consider the following example XHTML+SVG+MATHML document instance:
   * <pre>&lt;html xmlns="http://www.w3.org/1999/xhtml" xmlns:svg="http://www.w3.org/2000/svg"&gt;
    &lt;head&gt;
        &lt;title&gt;Sample XHTML with Equations&lt;/title&gt;
    &lt;/head&gt;
    &lt;body&gt;
        &lt;h1&gt;Sample MathML:&lt;/h1&gt;
        &lt;math xmlns="http://www.w3.org/1998/Math/MathML"&gt;
            &lt;mstyle displaystyle="false" scriptlevel="0"&gt;
                &lt;mrow&gt;
                    &lt;mfrac&gt;
                        &lt;mrow&gt;
                            &lt;mi mathcolor="gray"&gt;sin&lt;/mi&gt;
                            &lt;mo rspace="verythinmathspace"&gt;⁡&lt;/mo&gt;
                            &lt;mi&gt;θ&lt;/mi&gt;
                        &lt;/mrow&gt;
                        &lt;mi&gt;π&lt;/mi&gt;
                    &lt;/mfrac&gt;
                &lt;/mrow&gt;
            &lt;/mstyle&gt;
        &lt;/math&gt;
        &lt;h1&gt;Sample SVG:&lt;/h1&gt;
        &lt;svg:svg version="1.1" baseProfile="full" width="300px" height="200px"&gt;
            &lt;svg:circle cx="150px" cy="100px" r="50px" fill="#ff0000"
                    stroke="#000000" stroke-width="5px"/&gt;
        &lt;/svg:svg&gt;
    &lt;/body&gt;
&lt;/html&gt;</pre>

   * The resulting output will be:
   * <pre><b>Namespace 'http://www.w3.org/1998/Math/MathML' contains:</b>
&lt;math&gt; (1 occurrence)
&lt;mo&gt; (1 occurrence)
&lt;mrow&gt; (2 occurrences)
&lt;mfrac&gt; (1 occurrence)
&lt;mi&gt; (3 occurrences)
&lt;mstyle&gt; (1 occurrence)

<b>Namespace 'http://www.w3.org/1999/xhtml' contains:</b>
&lt;body&gt; (1 occurrence)
&lt;title&gt; (1 occurrence)
&lt;html&gt; (1 occurrence)
&lt;h1&gt; (2 occurrences)
&lt;head&gt; (1 occurrence)

<b>Namespace 'http://www.w3.org/2000/svg' contains:</b>
&lt;svg&gt; (1 occurrence)
&lt;circle&gt; (1 occurrence)</pre>
   */
  public void printStatistics () {
    for (final Entry<String, ElementcountByName> entry: frequenciesByNamespace.entrySet()) {
      System.out.println("Namespace '" + entry.getKey() + "' contains:");
      entry.getValue().printStatistics();
      System.out.println();
    }
  }
}